@@ -1,5 +1,12 @@
+import json
+
+from selenium.webdriver.support import expected_conditions as EC
+from selenium.webdriver.support.ui import WebDriverWait
+from selenium.webdriver.common.by import By
 from typing import Optional
+from threading import Thread
 import time
+import os
 
 def invoke_api_list(link: str, token: str, method: Optional[str] = "GET", headers: Optional[str] = None, print_response: Optional[bool] = False) -> dict:
     import requests
@@ -142,7 +149,8 @@ def invoke_api_proc(link: str, payload_vars: dict, token: str, method: str, prin
                 time.sleep(5)
                 continue
 
-            else: raise ValueError("Api proc final falhou")
+            else:
+                raise ValueError("Api proc final falhou")
 
 def invoke_api_proc_log(link, id_robo, token):
     import requests
@@ -163,13 +171,9 @@ def invoke_api_proc_log(link, id_robo, token):
     print(f"\n{responseinsert.json()}")
 
 def login_2fac(driver, certificate, system, token, code_timeout=60):
-    from . import mostrar_mensagem
-    import time
     import requests
     import pyautogui
-    from selenium.webdriver.common.by import By
-    from selenium.webdriver.support.ui import WebDriverWait
-    from selenium.webdriver.support import expected_conditions as EC
+    from . import mostrar_mensagem
 
     class login_2fac:
         def __init__(self):
@@ -179,7 +183,35 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
             self.code_timeout = code_timeout
             self.driver = driver
 
-    class tools(login_2fac):
+    class Pop_up_protection(login_2fac):
+        def __init__(self):
+            super().__init__()
+            self.status = False
+
+        def __monitor(self):
+            while self.status:
+
+                handles = self.driver.window_handles
+                if len(handles) > 1:
+                    self.driver.switch_to.window(self.driver.window_handles[-1])
+
+                try:
+                    alert = self.driver.switch_to.alert
+                    alert.accept()
+                except:
+                    pass
+
+                time.sleep(0.1)
+
+        def start(self):
+            self.status = True
+            protection = Thread(target=self.__monitor, daemon=True)
+            protection.start()
+
+        def stop(self):
+            self.status = False
+
+    class tool(login_2fac):
         def find_element_with_wait(self, by, value, timeout=10):
             global driver
             return WebDriverWait(
@@ -222,6 +254,7 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
                         time.sleep(5)
                         continue
                     else:
+                        self.driver.quit()
                         raise('Todas as tentativas falharam!')
 
         def invoke_update_status(self, id):
@@ -246,34 +279,118 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
                         time.sleep(5)
                         continue
                     else:
+                        self.driver.quit()
                         raise('Todas as tentativas falharam!')
 
     class whoom_codes(login_2fac):
         def __init__(self):
             super().__init__()
 
-        def codes_2_fac(self):
+        def extension_check(self):
 
-            self.extension_check()
+            self.driver.get('chrome-extension://lnidijeaekolpfeckelhkomndglcglhh/index.html')
             time.sleep(3)
 
+            for _ in range(10):
+
+                # Caso a extensão já esteja instalada
+                try:
+                    tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Digite ou selecione um sistema pra acessar"]', timeout=1)
+                    return
+                except: pass
+
+                try:
+                    tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Insira aqui o seu email"]', timeout=1)
+                    return
+                except: pass
+
+                # Caso a extensão não esteja instala
+                if 'This page has been blocked by Chrome' in driver.page_source:
+                    break
+
+                if 'eliezer@bcfox.com.br' in self.driver.page_source:
+                    tools.find_element_with_wait(By.XPATH, "//span[text()='alterar']").click()
+                    return
+
+            # Abrir uma nova aba
+            self.driver.execute_script("window.open('');")
+
+            # Fechar a aba original
+            self.driver.close()
+
+            # Mudar para a nova aba
+            self.driver.switch_to.window(self.driver.window_handles[-1])
+
+            time.sleep(1)
+
+            self.driver.get("https://chromewebstore.google.com/detail/whom-gerenciador-de-certi/lnidijeaekolpfeckelhkomndglcglhh")
+
             try:
-                tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Digite ou selecione um sistema pra acessar"]', timeout=10).send_keys(self.system)
+                tools.find_element_with_wait(By.XPATH, "//span[contains(text(), 'no Chrome') or contains(text(), 'Usar') or contains(text(), 'Add to Chrome')]").click()
+
+            except:
+                tools.find_element_with_wait(By.XPATH, "//span[contains(text(), 'do Chrome') or contains(text(), 'Usar') or contains(text(), 'Add to Chrome')]").click()
+                pyautogui.press('enter')
+                time.sleep(1)
+                tools.find_element_with_wait(By.XPATH, "//span[contains(text(), 'no Chrome') or contains(text(), 'Usar') or contains(text(), 'Add to Chrome')]").click()
+
+
+            time.sleep(5)
+            import pygetwindow as gw
+            windows = gw.getAllTitles()
+            for w in windows:
+                if w and "Whom?".lower() in w.lower():
+                    print("Janela de extensão detectada:", w)
+                    window = gw.getWindowsWithTitle(w)[0]
+                    window.activate()
+                    time.sleep(1)
+                    coords = (gw.getActiveWindow().left + gw.getActiveWindow().size[0] // 2,
+                              gw.getActiveWindow().top + int(gw.getActiveWindow().size[1] * 0.3))
+                    pyautogui.moveTo(coords[0], coords[1])
+                    pyautogui.click()
+                    
+                    # Envia TAB e ENTER do teclado físico
+                    pyautogui.press('tab')
+                    time.sleep(0.5)
+                    pyautogui.press('enter')
+                    time.sleep(5)
+                    break
+
+            self.driver.get('chrome-extension://lnidijeaekolpfeckelhkomndglcglhh/index.html')
+
+        def codes_2_fac(self):
+            try:
+                tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Digite ou selecione um sistema pra acessar"]', timeout=2).send_keys(self.system)
                 code_insertion = True
 
             except:
                 self.driver.get('chrome-extension://lnidijeaekolpfeckelhkomndglcglhh/index.html')
+
                 # Request the code
                 for _ in range(50):  # Wait the extension to load
 
                     try:
-                        tools.find_element_with_wait(By.XPATH, '//*[@id="root"]/div/div[1]/div//div//input', timeout=1).send_keys('eliezer@bcfox.com.br')
-
+                        tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Insira aqui o seu email"]', timeout=1).send_keys('eliezer@bcfox.com.br')
+                        time.sleep(1)
                         break
                     except:
                         self.driver.get('chrome-extension://lnidijeaekolpfeckelhkomndglcglhh/index.html')
 
-                tools.find_element_with_wait(By.XPATH, '//button').click()
+                # Envia o código pro email, o for é só para tratativa de bugs
+                for _ in range(10):
+                    try:
+                        tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Digite aqui o código que enviamos para o seu e-mail"]', timeout=1)
+                        break
+
+                    except:
+                        try:
+                            element = tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Insira aqui o seu email"]', timeout=1)
+                            element.clear()
+                            element.send_keys('eliezer@bcfox.com.br')
+                            tools.find_element_with_wait(By.XPATH, '//button').click()
+                            time.sleep(1)
+                        except:
+                            break
 
                 # Attempts the new codes until success or requests limit
                 for _ in range(code_timeout):
@@ -286,11 +403,19 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
                         CODE = response['CODIGO']
                         ID = response['ID']
 
-                        tools.find_element_with_wait(By.XPATH, '//input[@type="password"]').send_keys(CODE)
-                        tools.find_element_with_wait(By.XPATH, '//div/div[2]/button').click()
+                        element = tools.find_element_with_wait(By.XPATH, '//input[@type="password"]')
+                        element.clear()
+                        element.send_keys(CODE)
+
+                        for _ in range(10):
+                            try:
+                                tools.find_element_with_wait(By.XPATH, '//div/div[2]/button', timeout=2).click()
+                                time.sleep(1)
+                            except:
+                                break
 
                         # Check the code result
-                        for _ in range(7):
+                        for _ in range(30):
 
                             # Correct
                             try:
@@ -306,7 +431,7 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
                             # Wrong
                             try:
                                 tools.find_element_with_wait(By.XPATH, "//span[contains(text(), 'Senha inválida')]", timeout=1)
-                                tools.find_element_with_wait(By.XPATH, "//button[text()='Voltar']", timeout=1)
+                                tools.find_element_with_wait(By.XPATH, "//button[text()='Voltar']", timeout=1).click()
                                 code_insertion = False
                                 break
 
@@ -321,19 +446,23 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
                         break
 
             if not(code_insertion):
-                mostrar_mensagem('Código WHOOM não chegou dentro do timeout estabelecido')
+                self.driver.quit()
                 raise TimeoutError('Código WHOOM não chegou dentro do timeout estabelecido')
 
-            # Selects the system to access
+        def select_system(self):
+            time.sleep(4)
             try:
-                lines = tools.find_elements_with_wait(self.driver, By.XPATH, '//*[@id="root"]/div[2]/div/div[1]/div/div[2]/div/div/div', timeout=5)
+                tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Insira aqui o seu email"]',
+                                             timeout=1).click()
+                time.sleep(1)
+                self.codes_2_fac()
+                self.select_system()
+                return
             except:
-                try:
-                    lines = tools.find_elements_with_wait(self.driver, By.XPATH, '//*[@id="root"]/div[2]/div/div[2]/div/div[2]/div/div/div', timeout=5)
-                except Exception as e:
-                    mostrar_mensagem('Não conseguiu achar o sistema no certificado')
-                    print(f'{e}\n\n\n\n\n\n')
-                    raise ValueError('Não conseguiu achar o sistema no certificado') from e
+                pass
+
+            # Selects the system to access
+            lines = tools.find_elements_with_wait(By.XPATH, '//div[@role="menu"]//div[@role="menuitem"]')
 
             finded = False
             div_list = []
@@ -360,9 +489,11 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
 
             if not div_list:
                 mostrar_mensagem('Não conseguiu achar o sistema no certificado')
+                self.driver.quit()
                 raise ValueError('Não conseguiu achar o sistema no certificado')
 
             if len(div_list) == 1:
+                time.sleep(1)
                 div_list[0].click()
 
             # Just do this loop if there are more than one system
@@ -374,58 +505,60 @@ def login_2fac(driver, certificate, system, token, code_timeout=60):
 
                 if len(div_list) == 1:
                     div_list[0].click()
+                elif len(div_list) == 2:
+                    div_list[0].click()
                 else:
                     mostrar_mensagem('Mais de um sistema encontrado, verifique o nome do sistema no WHOOM e coloque um nome único na função')
+                    self.driver.quit()
                     raise ValueError('Mais de um sistema encontrado, verifique o nome')
 
-            # Verifies if the system was opened
-            num_windows = len(self.driver.window_handles)
-            for _ in range(5):
+            try:
+                tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Insira aqui o seu email"]',
+                                             timeout=1).click()
                 time.sleep(1)
-                if len(self.driver.window_handles) == num_windows:
-                    tools.find_element_with_wait(By.XPATH, "//button[text()='Acessar']").click()
+                self.codes_2_fac()
+                self.select_system()
+                return
+            except:
+                pass
+
+            # Verifies if the system was opened
+            for _ in range(30):
+                time.sleep(1)
+                if len(self.driver.window_handles) == 1:
+                    try:
+                        tools.find_element_with_wait(By.XPATH, "//button[text()='Acessar']", timeout=1).click()
+                    except:
+                        time.sleep(1)
                 else:
+                    time.sleep(3)
                     break
 
-        def extension_check(self):
+            self.driver.switch_to.window(self.driver.window_handles[-1])
+            attempt = 0
+            while 'whoom' in self.driver.title.strip().lower() and attempt <= 180:
+                time.sleep(1)
+                attempt += 1
+            time.sleep(5)
 
-            self.driver.get('chrome-extension://lnidijeaekolpfeckelhkomndglcglhh/index.html')
+            if attempt >= 180:
+                mostrar_mensagem('Whoom congelou no conectar com site.')
+                raise SystemError('Whoom congelou no conectar com site.')
 
-            try:
-                tools.find_element_with_wait(By.XPATH, '//input[@placeholder="Digite ou selecione um sistema pra acessar"]', timeout=10).send_keys(self.system)
-                return
-
-            except Exception as e:
-
-                # Abrir uma nova aba
-                self.driver.execute_script("window.open('');")
-
-                # Fechar a aba original
+            protection.stop()
+            if len(self.driver.window_handles) > 1:
+                self.driver.switch_to.window(self.driver.window_handles[0])
                 self.driver.close()
-
-                # Mudar para a nova aba
                 self.driver.switch_to.window(self.driver.window_handles[-1])
 
-                time.sleep(1)
-
-                self.driver.get("https://chromewebstore.google.com/detail/whom-gerenciador-de-certi/lnidijeaekolpfeckelhkomndglcglhh")
-
-                try:
-                    tools.find_element_with_wait(By.XPATH, "//span[contains(text(), 'Instalar') or contains(text(), 'Obter') or contains(text(), 'Add to Chrome')]").click()
-                except Exception as e:
-                    return
-                time.sleep(5)
-
-                # Envia TAB e ENTER do teclado físico
-                pyautogui.press('tab')
-                time.sleep(0.5)
-                pyautogui.press('enter')
-                time.sleep(5)
-
-                self.driver.get('chrome-extension://lnidijeaekolpfeckelhkomndglcglhh/index.html')
-
-    tools = tools()
+    # Instances
+    tools = tool()
     api = invokes_whoom()
+    protection = Pop_up_protection()
     bot = whoom_codes()
 
+    # Operacional
+    bot.extension_check()
+    protection.start()
     bot.codes_2_fac()
+    bot.select_system()
\ No newline at end of file
